	page	60,132

	Title	LCD_EX.COM lcd driver
	Subttl	Copyright JK microsystems 1997-2001

;************************************************************************;
;*								       	*;
;*	Copyright JK microsystems 1997-2001	-  All Rights Reserved	  	*;
;*									*;
;*	This software is NOT shareware, freeware, or public domain.	*;
;*	It is the property of JK microsystems.				*;
;*									*;
;*	Customers of JK microsystems may modify the source code		*; 
;*	and/or distribute the binary image of this software without 	*;
;*	additional costs provided it is run only on hardware 		*;
;*	manufactured by JK microsystems.  All other use is expressly 	*;
;*	prohibited.							*;
;*									*;
;*	Update Log							*;
;*									*;
;*	Version     Date        Comments                            Progammer     *;
;*	-------	-------	-------------------------------	----------	  *;
;*	1.0         7-16-95	First release	 		      jds		  *;
;*	2.0         5-18-96     Corrected timing error				        *;
;*			            Now works with Seiko LCDs           jds		  *;
;*	3.0         12-17-97    Version for 386ex                   jds		  *;
;*	3.1         5-19-99     Fix for reset with Winbond lcd      jds/Rob Peter *;
;*    3.2         10-2-01     Copy init code from LCD_LF.                       *;
;*                            Should resolve warm boot probs.     ew            *;
;********************************************************************************;

Code_seg	Segment
	assume	cs:code_seg,ds:code_seg,ss:code_seg

	org	100h
start:					; Put the initialization code at
	jmp	init			; the end so we don't have to save it
	
;-----------------------------------------------------------------------------
;
;	This is invoked with each bios LPT interrupt
;
LPT_int:
	sti				; interrupts back on
	cmp	dx,1			; LPT number greater than 2
	jna	do_lcd			; no, do it
	jmp	chain			; yes, chain to old handler

do_lcd:
	push	dx			; save a couple registers
	push	bx

	cmp	dx,1			; LPT2?
	je	LPT2			; yes, go there
		
LPT1:
	mov	ah,0			; set lcd0
	cmp	al,0A0h			; command escape char?
	jne	LPT1_not_esc		; no, go around
	mov	cs:[LPT1_cmd_flag],1	; yes, set the flag
	jmp	lpt_done		; and return

LPT1_not_esc:
	mov	bl,al			; save the char
	shr	al,1			; move the first nibble over	
	shr	al,1
	shr	al,1
	shr	al,1
	cmp	cs:[lpt1_cmd_flag],1	; was it a command?
	je	send_LPT1_cmd		; yes, go elsewhere to send it
	call	data_write		; no, it's data so send it here
	mov	al,bl			; restore the data
	call	data_write		; send the low nibble
	jmp	lpt_done		; and leave

send_LPT1_cmd:
	call	cmd_write		; come here to send it as a command
	mov	al,bl			; restore the data
	call	cmd_write		; and send the low nibble
	mov	cs:[lpt1_cmd_flag],0	; clear the flag
	jmp	lpt_done		; and leave

LPT2:
	mov	ah,1			; set lcd1
	cmp	al,0A0h			; command escape char?
	jne	LPT2_not_esc		; no, go around
	mov	cs:[LPT2_cmd_flag],1	; yes, set the flag
	jmp	lpt_done		; and return

LPT2_not_esc:
	mov	bl,al			; save the char
	shr	al,1			; move the first nibble over	
	shr	al,1
	shr	al,1
	shr	al,1
	cmp	cs:[lpt2_cmd_flag],1	; was it a command?
	je	send_LPT2_cmd		; yes, go elsewhere to send it
	call	data_write		; no, it's data so send it here
	mov	al,bl			; restore the data
	call	data_write		; send the low nibble
	jmp	lpt_done		; and leave

send_LPT2_cmd:
	call	cmd_write		; come here to send it as a command
	mov	al,bl			; restore the data
	call	cmd_write		; and send the low nibble
	mov	cs:[lpt2_cmd_flag],0	; clear the flag
		
LPT_done:
	mov	ah,0D0h			; return good status
	pop	bx
	pop	dx			
	iret

chain:
	push	cs:[old_LPT_seg]	; pushing these and leaving with
	push	cs:[old_LPT_off]	; a retf will chain to old lpt
	retf				; routine


;----------------------------------------------------------------------------
;
;	Delay for 5ms
;
delay_5_ms:
	push 	ax			; ax gets trashed
	mov 	ax,1000			; time constant for 5ms

delay_5_ms_loop:
	dec	ax			; do the loop
	jnz	delay_5_ms_loop
	pop	ax
	ret
		
;----------------------------------------------------------------------------
;
;	Configure bits 0-3 of port D as an input port so that we
;	can read data from the lcd
;
bus_read:
	push	ax
	push	dx
	mov	dx,PortCtl		
	in	al,dx
	and	al,0FEh
	out	dx,al
	pop	dx
	pop	ax
	ret

;----------------------------------------------------------------------------
;
;	Configure bits 0-3 of port D as an output port so that we
;	can write data to the lcd
;
bus_write:
	push	ax
	push	dx
	mov	dx,PortCtl		
	in	al,dx
	or	al,1
	out	dx,al
	pop	dx
	pop	ax
	ret

;----------------------------------------------------------------------------
;
;	Set the bus to idle state
;
bus_idle:
	push	ax
	push	dx
	mov	dx,PortE		; port E
	mov	al,1			; idle state
	out	dx,al			; do it
	call	delay_5_ms	
	pop	dx
	pop	ax
	ret

;----------------------------------------------------------------------------
;
;	Short Delay
;
short_delay:
	push	ax
	pop	ax
	ret

;----------------------------------------------------------------------------
;
;	Write a command to the lcd specified in ah
;
cmd_write:
	call	bus_write		; set the bus direction to write
	call	short_delay

	mov	dx,PortD		; point to lcd data bus
	out	dx,al			; put AL 0-3 on the bus

	mov	dx,PortE		; point to lcd control lines
	mov	al,0			; drop the WR/ line, RS
	out	dx,al			; send it
	call	short_delay

	cmp	ah,0			; lcd 0?
	je	cmd_write_0		; yes, go there
	or	al,8			; no, set bit 3		
	jmp	raise_E
cmd_write_0:
	or	al,4			; lcd 1, set bit 2

raise_e:	
	out	dx,al			; raise E0 or E1
	call	short_delay
	and	al,0F3h			; drop E0 and E1
	out	dx,al
	call	short_delay

	call	bus_idle		; restore the bus to idle
	call	delay_5_ms		; wait for lcd to do it
	ret

;----------------------------------------------------------------------------
;
;	Write data to the lcd specified in ah
;
data_write:
	call	bus_write		; set the bus direction to write
	call	short_delay

	mov	dx,PortD		; point to lcd data bus
	out	dx,al			; put AL on the bus

	mov	dx,PortE		
	mov	al,2			; drop the WR/ line, raise RS
	out	dx,al			; send it
	call	short_delay

	cmp	ah,0			; lcd 0?
	je	data_write_0		; yes, go there
	or	al,8			; no, set bit 3		
	jmp	raise_E_data
data_write_0:
	or	al,4			; lcd 1, set bit 2

raise_e_data:	
	out	dx,al			; raise E0 or E1
	call	short_delay
	and	al,0F3h			; drop E0 and E1
	out	dx,al
	call	short_delay





;	call	bus_idle		; restore the bus to idle
;	call	delay_5_ms		; wait for lcd to do it
	ret

;-----------------------------------------------------------------------------
;
;	Data and equates	
;
old_LPT_seg	dw	0		; save the old LPT routine
old_LPT_off	dw	0		; vectors in case we need them

LPT1_cmd_flag	db	0		; 1 if next byte is a command
LPT2_cmd_flag	db	0		;

PortCtl		equ	word ptr 0FA00h	; direction control port
PortD		equ	word ptr 0FA02h	; port D, bits 0-3 used for LCD data
PortE		equ	word ptr 0FA03h ; port E, LCD control signals


last_add:				; last address to save when terminating

;-----------------------------------------------------------------------------
;
;	This is our initialization code
;
init:

	mov	ax,40h			; point to bios data area
	mov	ds,ax

	mov	ax,ds:[10h]		; get equipment word
	or	ah,010000000B		; bits 14 and 15 = number of printers
	and	ah,010111111B		; set bit 15, clear bit 14, 2 printers
	mov	ds:[10h],ax		; put equipment word back

	mov	ax,0			; point to interrupt vector table
	mov 	ds,ax

	cli				; no ints while changing vectors

	mov	ax,ds:[5Ch]		; get int 17h offset
	mov	cs:[old_LPT_off],ax	; save it
	mov	ds:[5Ch],offset LPT_int ; and replace it with our routine
	mov	ax,ds:[5Eh]		; get int 17h segment
	mov	cs:[old_LPT_seg],ax	; save it
	mov	ax,cs			; get our segment
	mov	ds:[5Eh],ax		; and install it

	sti				; ints back on

	mov	dx,portctl		; set control bus to output
	in	al,dx
	or	al,00001110b
	out	dx,al

	call	bus_idle		; set bus_idle state

; ---------- LCD init code starts here

	mov	cx,3			; send 3 03 commands to lcd 0
init_loop0:
	call	delay_5_ms
	mov	ah,0
	mov	al,3
	call	cmd_write
	loop	init_loop0

	mov	cx,3			; send 3 03 commands to lcd 1
init_loop1:
	call	delay_5_ms
	mov	ah,1
	mov	al,3
	call	cmd_write
	loop	init_loop1

	call	delay_5_ms		; give it some time to settle
	mov	ah,0			; lcd 0
	mov	al,2			; set to 4 bit bus
	call	cmd_write		; write it

	call	delay_5_ms		; give it some time to settle
	mov	ah,1			; lcd 1
	mov	al,2			; set to 4 bit bus
	call	cmd_write		; write it

	mov	cx,8			; init string is 8 bytes
	mov	di,offset init_string	; clear the lcd and home the cursor
clear_loop0:
	mov	ah,0			; lcd 0
	mov	al,cs:[di]
	int	17h
	inc	di
	loop 	clear_loop0

	mov	cx,8			; init string is 8 bytes
	mov	di,offset init_string	; clear the lcd and home the cursor
clear_loop1:
	mov	ah,1			; lcd 1	
	mov	al,cs:[di]
	int	17h
	inc	di
	loop 	clear_loop1

; ---------- LCD init code ends here


	mov	ax,3100h		; TSR function code
	mov 	dx,offset last_add 	; get last address used
	mov 	cl,4			; convert to paragraphs
	shr     dx,cl			; by dividing by 16
	inc	dx			; and rounding up by one

	int	21h			; tell dos to do it

init_string:
	db	160,40			; 2 lines
 	db	160,12			; display on
	db	160,6			; position increment, no shift
	db	160,1			; clear screen, home cursor


code_seg 	ends

	end	start
